﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.IO.Ports;
using System.Timers;

namespace FraCon
{
    public partial class Form1 : Form
    {
        private const int statusOK = 0;
        private const int statusNG = -1;
        private const int algoInstNum = 0;
        private const int algoStrevNum = 1;
        private const int algoStdelayNum = 2;
        private const int algoStchoNum = 3;
        private const int algoStflgNum = 4;
        private const int algoStphsNum = 5;
        private const int algoStwahNum = 6;

        private const int dsp1PatchNum = 80;
        private const int dsp2PatchNum = 80;
        private const int dspMemoryGroupValue = 8;
        private const int dspMemoryNumValue = 40;
        private const int dspAlgoNum = 7;
        private const string aFrameComDeviceName = "Silicon Labs CP210x USB to UART Bridge ";

        private string aFramePortName; // port name.
        private bool isSerialTimerout;
        private System.Timers.Timer timer; // for serial transfer.
        private List<ProjectGroupList> prjGrpList;
        private ProjectToneNameList prjToneNameList;
        private ProjectToneDataList prjToneDataList;

        private Button[] swCollection;
        private bool isSerialConnected = false;
        private bool isEncoderAdjActive = false;
        private int swPatternPre = 0x0;

        /***** Form *****/
        public Form1()
        {
            InitializeComponent();
            makeControlCollection();
            InitComPort();
            prjGrpList = new List<ProjectGroupList>();
            prjToneNameList = new ProjectToneNameList();
            prjToneDataList = new ProjectToneDataList();
            timer_sync.Enabled = true;
        }

        /***** UI control *****/
        private void makeControlCollection()
        {
            swCollection = (Button[])GetControlArrayByName(this, "button_");
        }

        public object GetControlArrayByName(Form frm, string name)
        {
            System.Collections.ArrayList ctrs =
                new System.Collections.ArrayList();
            object obj;
            for (int i = 0;
                (obj = FindControlByFieldName(frm, name + i.ToString())) != null;
                i++)
                ctrs.Add(obj);
            if (ctrs.Count == 0)
                return null;
            else
                return ctrs.ToArray(ctrs[0].GetType());
        }

        public static object FindControlByFieldName(Form frm, string name)
        {
            System.Type t = frm.GetType();

            System.Reflection.FieldInfo fi = t.GetField(
                name,
                System.Reflection.BindingFlags.Public |
                System.Reflection.BindingFlags.NonPublic |
                System.Reflection.BindingFlags.Instance |
                System.Reflection.BindingFlags.DeclaredOnly);

            if (fi == null)
                return null;

            return fi.GetValue(frm);
        }

        /***** API function *****/
        private void setEncoderValue(int sel)
        {
            List<string> aFrameReply = new List<string>();
            int status;

            writeData("aFG3\r\n");
            status = waitAndGetLines(ref aFrameReply);
            if (status == statusOK)
            {
                int pos = int.Parse(aFrameReply[0]);
                aFrameReply.Clear();
                int posNew;
                if (sel == 0) // down
                {
                    posNew = pos - 1;
                }
                else // up
                {
                    posNew = pos + 1;
                }
                writeData("aFS3:" + posNew.ToString() + "\r\n");
                status = waitAndGetLines(ref aFrameReply);
                if (status == statusOK)
                {
                    if (int.Parse(aFrameReply[0]) != 0)
                    {
#if DEBUG
                        MessageBox.Show("[ERR] SetEncoder --- reply != 0");
#endif
                    }
                }
                else
                {
#if DEBUG
                    MessageBox.Show("[ERR] SetEncoder --- statusNG: ");
#endif
                }
            }
            else
            {
#if DEBUG
                MessageBox.Show("[ERR] GetEncoder --- statusNG: ");
#endif
            }
        }

        private void getProjectGroupList()
        {
            for (int i = 0; i < dspMemoryGroupValue; i++)
            {
                ProjectGroupList grpList = new ProjectGroupList();
                List<string> aFrameReply = new List<string>();
                int status;

                writeData("aFGD:" + i.ToString() + "\r\n");
                status = waitAndGetLines(dspMemoryNumValue + 1, ref aFrameReply);
                if (status == statusOK)
                {
                    foreach (string line in aFrameReply)
                    {
                        if (line.Split(':')[0] == "Gmax")
                        {
                            grpList.gMax = int.Parse(line.Split(':')[1]);
                        }
                        else
                        {
                            string[] values = line.Split(',');
                            grpList.instNum.Add(int.Parse(values[0]));
                            grpList.effectNum.Add(int.Parse(values[1]));
                        }
                    }
                    prjGrpList.Add(grpList);
                }
                else // NG
                {
#if DEBUG
                    MessageBox.Show("[ERR] GetProjectGroupList");
#endif
                }
            }
        }

        private void getProjectToneNameList()
        {
            for (int i = 0; i < 2; i++)
            {
                List<string> aFrameReply = new List<string>();
                int status;

                writeData("aFGE:" + i.ToString() + "\r\n");
                if (i == 0) // Inst
                {
                    status = waitAndGetLines(dsp1PatchNum, ref aFrameReply);
                    if (status == statusOK)
                    {
                        foreach (string line in aFrameReply)
                        {
                            prjToneNameList.instName.Add(line);
                        }
                    }
                    else // NG
                    {
#if DEBUG
                        MessageBox.Show("[ERR] GetProjectToneNameList");
#endif
                    }
                }
                else // Effect
                {
                    status = waitAndGetLines(dsp2PatchNum, ref aFrameReply);
                    if (status == statusOK)
                    {
                        foreach (string line in aFrameReply)
                        {
                            prjToneNameList.effectName.Add(line);
                        }
                    }
                    else // NG
                    {
#if DEBUG
                        MessageBox.Show("[ERR] GetProjectToneNameList");
#endif
                    }
                }
            }
        }

        private void getProjectToneDataList()
        {
            for (int i = 0; i < 2; i++)
            {
                List<string> aFrameReply = new List<string>();
                int status;

                if (i == 0) // Inst
                {
                    writeData("aFGF:" + i.ToString() + ",-1\r\n");
                    status = waitAndGetLines(dsp1PatchNum * 4, ref aFrameReply);
                    if (status == statusOK)
                    {
                        getProjectToneData(ref prjToneDataList.instToneData, ref aFrameReply);
                    }
                    else // NG
                    {
#if DEBUG
                        MessageBox.Show("[ERR] GetProjectToneDataList --- statusNG");
#endif
                    }
                }
                else // Effect
                {
                    writeData("aFGF:" + i.ToString() + ",-1\r\n");
                    status = waitAndGetLines(dsp2PatchNum * 4, ref aFrameReply);
                    if (status == statusOK)
                    {
                        getProjectToneData(ref prjToneDataList.effectToneData, ref aFrameReply);
                    }
                    else // NG
                    {
#if DEBUG
                        MessageBox.Show("[ERR] GetProjectToneDataList --- statusNG");
#endif
                    }
                }
            }
        }

        private void getProjectToneData(ref List<ProjectToneData> toneDataList, ref List<string> replyList)
        {
            for (int i = 0; i < replyList.Count; i += 4)
            {
                ProjectToneData toneData = new ProjectToneData();
                int paramNum = 0;
                toneData.algo = int.Parse(replyList[i]);  // Algo
                toneData.name = replyList[i + 1];             // Name
                paramNum = int.Parse(replyList[i + 2]);       // PNum
                toneData.pNum = paramNum;

                string[] paramStr = replyList[i + 3].Split(',');
                List<string> paramList = new List<string>();
                paramList.AddRange(paramStr);
                paramList.RemoveAt(paramList.Count - 1); // Last factor is "". and Remove.

                if (paramList.Count == paramNum)
                {
                    foreach (string valStr in paramList)
                    {
                        toneData.pVal.Add(int.Parse(valStr));
                    }
                }
                else
                {
#if DEBUG
                    MessageBox.Show("[ERR] getProjectToneData --- param number error");
#endif
                }
                toneDataList.Add(toneData);
            }
        }

        private Int16 getSwitch()
        {
            List<string> aFrameReply = new List<string>();
            int status;
            Int16 pattern = 0x0000;

            if (isSerialConnected == false)
                return 0;

            writeData("aFG2\r\n");
            status = waitAndGetLines(ref aFrameReply);
            if (status == statusOK)
            {
                pattern = Convert.ToInt16(aFrameReply[0], 16);
            }
            else
            {
#if DEBUG
                MessageBox.Show("[ERR] GetSwitch --- statusNG: ");
#endif
            }
            return pattern;
        }

        private void getLEDandUpdate()
        {
            List<string> aFrameReply = new List<string>();
            int status;

            if (isSerialConnected == false)
                return;

            writeData("aFG4\r\n");
            status = waitAndGetLines(ref aFrameReply);
            if (status == statusOK)
            {
                int sel;
                int ledPattern_L = Convert.ToInt32(aFrameReply[0], 16);
                int ledPattern_R = ledPattern_L >> 16;
                int ledPattern_Peak = ledPattern_L >> 12;
                int ledPattern_Enc = ledPattern_L >> 28;

                for (int i = 1; i <= 5; i++) // Left
                {
                    sel = ledPattern_L & 0x3;
                    swCollection[i].BackColor = getLEDColor(sel);
                    ledPattern_L >>= 2;
                }
                for (int i = 6; i <= 10; i++) // Right
                {
                    sel = ledPattern_R & 0x3;
                    swCollection[i].BackColor = getLEDColor(sel);
                    ledPattern_R >>= 2;
                }
                // Encoder
                sel = ledPattern_Enc & 0x3;
                swCollection[12].BackColor = getLEDColor(sel);
                // Peak
                for (int i = 0; i < 2; i++)
                {
                    sel = ledPattern_Peak & 0x3;
                    if (i == 0) // Center
                    {
                        label_peak_center.BackColor = getLEDColor(sel);
                    }
                    else // Edge
                    {
                        label_peak_edge.BackColor = getLEDColor(sel);
                    }
                    ledPattern_Peak >>= 2;
                }
            }
            else // NG
            {
#if DEBUG
                MessageBox.Show("[ERR] GetLED");
#endif
            }
        }

        private string getLcdString()
        {
            string retStr = null;
            List<string> aFrameReply = new List<string>();
            int status;

            if (isSerialConnected == false)
                return "";

            writeData("aFG5:0,16\r\n");
            status = waitAndGetLines(ref aFrameReply);
            if (status == statusOK)
            {
                if (aFrameReply[0].IndexOf('?') >= 0)
                {
                    retStr = aFrameReply[0].Replace('?', ' ') + "\r\n"; // Upper
                }
                else
                {
                    retStr = aFrameReply[0] + "\r\n"; // Upper
                }
            }
            else // NG
            {
#if DEBUG
                MessageBox.Show("[ERR] GetLCD");
#endif
            }
            aFrameReply.Clear();
            writeData("aFG5:32,16\r\n");
            status = waitAndGetLines(ref aFrameReply);
            if (status == statusOK)
            {
                if (aFrameReply[0].IndexOf('?') >= 0)
                {
                    retStr += aFrameReply[0].Replace('?', ' ') + "\r\n"; // Lower
                }
                else
                {
                    retStr += aFrameReply[0]; // Lower
                }
            }
            else // NG
            {
#if DEBUG
                MessageBox.Show("[ERR] GetLCD");
#endif
            }
            return retStr;
        }

        private void setSwitchPush(int index)
        {
            List<string> aFrameReply = new List<string>();
            int status;
            string[] tempStr = swMaskAndName[index].Split(':');
            string swDataStr = tempStr[0];
            string swName = tempStr[1];


            writeData("aFS2:" + swDataStr + "\r\n"); // Push
            status = waitAndGetLines(ref aFrameReply);
            if (status == statusOK)
            {
                if (int.Parse(aFrameReply[0]) != 0)
                {
#if DEBUG
                    MessageBox.Show("[ERR] SetSwitch --- reply != 0");
#endif
                }
            }
            else
            {
#if DEBUG
                MessageBox.Show("[ERR] SetSwitch --- statusNG: " + swName);
#endif
            }
        }

        private void setSwitchRelease(int index)
        {
            List<string> aFrameReply = new List<string>();
            int status;

            writeData("aFS2:0x0000\r\n");   // Release
            status = waitAndGetLines(ref aFrameReply);
            if (status == statusOK)
            {
                if (int.Parse(aFrameReply[0]) != 0)
                {
#if DEUBG
                    MessageBox.Show("[ERR] SetSwitch --- reply != 0");
#endif
                }
            }
            else
            {
#if DEBUG
                MessageBox.Show("[ERR] SetSwitch --- statusNG: ");
#endif
            }
        }

        /***** other *****/
        private Color getLEDColor(int sel)
        {
            Color color;
            switch (sel)
            {
                case 1:
                    color = Color.Red;
                    break;
                case 2:
                    color = Color.GreenYellow;
                    break;
                case 3:
                    color = Color.Gold;
                    break;
                case 0:
                default:
                    color = Color.Transparent;
                    break;
            }
            return color;
        }

        private int waitAndGetLines(int lineNum, ref List<string> replyData)
        {
            int status = 0;
            int rcvLineNum = 0;
            string rcvStr = "";

            while (rcvLineNum < lineNum)
            {
                isSerialTimerout = false;
                timer.Start();
                while (isSerialRcved == false)
                {
                    if (isSerialTimerout == true)
                        break;
                }
                timer.Stop();
                isSerialRcved = false;
                if (isSerialTimerout == false)
                {
                    string str = getStringfromRcvdData();
                    if (str != null)
                    {
                        rcvLineNum += str.Length - str.Replace("\n", "").Length;
                        rcvStr += str;
                    }
                    else // null
                    {
                        status = -2;
                        break;
                    }
                }
                else // Timeout
                {
                    status = -1;
                    break;
                }
            }
            replyData.AddRange(rcvStr.Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries));
            return status;
        }

        private int waitAndGetLines(ref List<string> replyData)
        {
            int status = 0;

            isSerialTimerout = false;
            timer.Start();
            while (isSerialRcved == false)
            {
                if (isSerialTimerout == true)
                    break;
            }
            timer.Stop();
            isSerialRcved = false;
            if (isSerialTimerout == false)
            {
                string[] lines = getStringArrayfromRcvdData();
                if (lines != null)
                {
                    replyData.AddRange(lines);
                }
                else // null
                {
                    status = -2;
                }
            }
            else // Timeout
            {
                status = -1;
            }
            return status;
        }

        private string getParamString(int algo, int idx, int value)
        {
            string retStr;

            switch (algo)
            {
                case algoStrevNum:
                    retStr = strevChLabel[idx] + ":" + getStrevValue(idx, value);
                    break;
                case algoStdelayNum:
                    retStr = stdelayChLabel[idx] + ":" + getStdelayValue(idx, value);
                    break;
                case algoStchoNum:
                    retStr = stchousChLabel[idx] + ":" + getStchorusValue(idx, value);
                    break;
                case algoStflgNum:
                    retStr = stflangerChLabel[idx] + ":" + getStflangerValue(idx, value);
                    break;
                case algoStphsNum:
                    retStr = stphaserChLabel[idx] + ":" + getStphaserValue(idx, value);
                    break;
                case algoStwahNum:
                    retStr = stwahChLabel[idx] + ":" + getStwahValue(idx, value);
                    break;
                case algoInstNum:
                default:
                    retStr = atdaChLabel[idx] + ":" + getAtdaValue(idx, value);
                    break;
            }
            return retStr;
        }

        private string getGroupHeadStrings(int grp, int max)
        {
            string grpStr;
            string retStr;

            if (grp < 4)
            {
                grpStr = (grp + 0xA).ToString("X") + "  ";
            }
            else
            {
                grpStr = ((grp & 0x3) + 0xA).ToString("X") + "' ";
            }
            retStr = "| GRP " + grpStr + "List Max:" + max.ToString().PadRight(27) + "|";
            return retStr;
        }

        private string getGroupDataStrings(int grp, int num)
        {
            string retStr;
            string grpAndNumStr = getCurGrpAndNumString(grp, num);
            string instStr = "I" + (prjGrpList[grp].instNum[num] + 1).ToString("00") + ":" + prjToneDataList.instToneData[prjGrpList[grp].instNum[num]].name;
            string effectStr = "E" + (prjGrpList[grp].effectNum[num] + 1).ToString("00") + ":" + prjToneDataList.effectToneData[prjGrpList[grp].effectNum[num]].name;
            retStr = grpAndNumStr + " | " + instStr.PadRight(17) + "| " + effectStr.PadRight(17) + "|";
            return retStr;
        }

        private string getCurGrpAndNumString(int grp, int num)
        {
            string grpAndNumStr;

            if (grp < 4)
            {
                grpAndNumStr = (grp + 0xA).ToString("X") + "-" + (num + 1).ToString("00");
            }
            else
            {
                grpAndNumStr = ((grp & 0x3) + 0xA).ToString("X") + "'" + (num + 1).ToString("00");
            }
            return grpAndNumStr;
        }

        /**************************************************/
        /*                 Event handler                  */
        /**************************************************/
        /***** Click *****/
        private void toolStripButton_Con_Click(object sender, EventArgs e)
        {
            List<string> aFrameReply = new List<string>();
            int status;

            timer = new System.Timers.Timer();
            timer.Elapsed += new ElapsedEventHandler(OnElapsedTimer);
            timer.Interval = 5000; // 5sec

            connectComPort();
            writeData("aFG0\r\n"); // Get Version
            status = waitAndGetLines(ref aFrameReply);
            if (status == statusOK)
            {
                label_Version.Text = aFrameReply[0].Split('-')[0]; // Extract version number.
            }
            else
            {
                MessageBox.Show("This aFrame version don't support USB Serial.");   // Timeout
                label_Version.Text = "VER.X.XX";
            }
            getLEDandUpdate();
            label_LCD.Text = getLcdString();

            toolStripButton_Discon.Enabled = true;
            toolStripButton_Con.Enabled = false;
            toolStripButton_Print.Enabled = true;

            timer_sync.Enabled = true;  // Start Timer event.
        }

        private void toolStripButton_Discon_Click(object sender, EventArgs e)
        {

            timer.Dispose();
            timer_sync.Enabled = false; // Stop Timer event.
            disconnectComPort();
            label_Version.Text = "VER.X.XX";
            label_LCD.Text = "XXXXX XXXXX";
            for (int i = 1; i < swCollection.Length; i++)
            {
                if (i == 11)
                    continue;
                swCollection[i].BackColor = Color.Transparent;
            }
            label_peak_center.BackColor = Color.Transparent;
            label_peak_edge.BackColor = Color.Transparent;
            prjGrpList.Clear();
            prjToneDataList.instToneData.Clear();
            prjToneDataList.effectToneData.Clear();
            prjToneNameList.instName.Clear();
            prjToneNameList.effectName.Clear();
            toolStripButton_Discon.Enabled = false;
            toolStripButton_Con.Enabled = true;
            toolStripButton_Print.Enabled = false;
        }

        private void toolStripButton_Print_Click(object sender, EventArgs e)
        {
            DialogResult result = saveFileDialog_printFile.ShowDialog();

            if (result == DialogResult.OK)
            {
                string filePath = saveFileDialog_printFile.FileName;
                StreamWriter sw = new StreamWriter(filePath, false, Encoding.GetEncoding("UTF-8"));
                List<string> aFrameReply = new List<string>();
                int status;
                int grp = 0, num = 0;
                string grpAndNum = "";
                string[] filePathArray = filePath.Split('\\');
                sw.WriteLine("<< " + filePathArray[filePathArray.Length - 1] + " >>");
                // Write Current Group Info
                sw.WriteLine("==============================================");
                writeData("aFGA\r\n");
                status = waitAndGetLines(ref aFrameReply);
                if (status == statusOK)
                {
                    string[] curToneNum = aFrameReply[0].Split(',');
                    if (curToneNum.Length == 5)
                    {
                        grp = int.Parse(curToneNum[0]);
                        num = int.Parse(curToneNum[1]);
                        int max = int.Parse(curToneNum[2]);
                        grpAndNum = getCurGrpAndNumString(grp, num);

                        sw.WriteLine("| GRP " + grpAndNum + "/" + max.ToString() + " (Current GRP Bank-Num/Max)     |");
                        sw.WriteLine("==============================================");
                    }
                }
                else // NG
                {
#if DEBUG
                    MessageBox.Show("[ERR] GetCurrentGroupToneNum");
#endif
                }

                // Write Current Tone Info
                string instName = "", effectName = "";
                aFrameReply.Clear();
                writeData("aFGB\r\n");
                status = waitAndGetLines(2, ref aFrameReply);
                if (status == statusOK)
                {
                    instName = aFrameReply[0];
                    effectName = aFrameReply[1];
                    sw.WriteLine("| " + grpAndNum + " | " + instName.PadRight(17) + "| " + effectName.PadRight(17) + "|");
                    sw.WriteLine("+------+------------------+------------------+");
                    sw.WriteLine("");
                }
                else // NG
                {
#if DEBUG
                    MessageBox.Show("[ERR] GetCurrentToneName");
#endif
                }

                getProjectGroupList();      // Get Project Group List
                //getProjectToneNameList();   // Get Project Tone Name List --- not use, use Tone Data List.
                getProjectToneDataList();   // Get Project Tone Data List

                // Write Grp List
                for (int i = 0; i < dspMemoryGroupValue; i++)
                {
                    sw.WriteLine("==============================================");
                    sw.WriteLine(getGroupHeadStrings(i, prjGrpList[i].gMax));
                    sw.WriteLine("==============================================");
                    for (int j = 0; j < prjGrpList[i].gMax; j++)
                    {
                        string str = "";
                        if ((i == grp) && (j == num))
                        {
                            str += "|*";
                        }
                        else
                        {
                            str += "| ";
                        }
                        sw.WriteLine(str + getGroupDataStrings(i, j));
                    }
                    sw.WriteLine("+------+------------------+------------------+");
                    sw.WriteLine("");
                }

                // Write Tone List
                for (int i = 0; i < dsp1PatchNum; i++) // Inst
                {
                    ProjectToneData toneData = prjToneDataList.instToneData[i];
                    sw.WriteLine("===========================");
                    sw.WriteLine("| I" + (i + 1).ToString("00") + ":" + toneData.name.PadRight(20) + "|");
                    sw.WriteLine("===========================");
                    sw.WriteLine("| ALGO | " + chDspAlgoIDtxt[toneData.algo].PadRight(17) + "|" + toneData.algo.ToString().PadLeft(7) + ",");
                    sw.WriteLine("| PNUM | " + toneData.pNum.ToString().PadLeft(2) + "parameters     |" + toneData.pNum.ToString().PadLeft(7) + ",");
                    sw.WriteLine("+------+------------------+");
                    for (int j = 0; j < toneData.pNum; j++)
                    {
                        sw.WriteLine("| P-" + (j + 1).ToString("00") + " | " + getParamString(toneData.algo, j, toneData.pVal[j]).PadRight(17)
                            + "|" + toneData.pVal[j].ToString().PadLeft(7) + ",");
                    }
                    sw.WriteLine("+------+------------------+");
                    sw.WriteLine("");
                }

                for (int i = 0; i < dsp2PatchNum; i++)  // Effect
                {
                    ProjectToneData toneData = prjToneDataList.effectToneData[i];
                    sw.WriteLine("===========================");
                    sw.WriteLine("| E" + (i + 1).ToString("00") + ":" + toneData.name.PadRight(20) + "|");
                    sw.WriteLine("===========================");
                    sw.WriteLine("| ALGO | " + chDspAlgoIDtxt[toneData.algo].PadRight(17) + "|" + toneData.algo.ToString().PadLeft(7) + ",");
                    sw.WriteLine("| PNUM | " + toneData.pNum.ToString().PadLeft(2) + "parameters     |" + toneData.pNum.ToString().PadLeft(7) + ",");
                    sw.WriteLine("+------+------------------+");
                    for (int j = 0; j < toneData.pNum; j++)
                    {
                        sw.WriteLine("| P-" + (j + 1).ToString("00") + " | " + getParamString(toneData.algo, j, toneData.pVal[j]).PadRight(17)
                            + "|" + toneData.pVal[j].ToString().PadLeft(7) + ",");
                    }
                    sw.WriteLine("+------+------------------+");
                    sw.WriteLine("");
                }
                sw.Close();
            }
        }

        private void button_mute_Click(object sender, EventArgs e)
        {
            if (serialPort_aFrame.IsOpen == false)
            {
                MessageBox.Show("Check connection !!", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            setSwitchPush(13);
            setSwitchRelease(13);
        }

        /***** Mouse Down/Up *****/
        private void button_MouseDown(object sender, MouseEventArgs e)
        {
            if (serialPort_aFrame.IsOpen == false)
            {
                MessageBox.Show("Check connection !!", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (e.Button == MouseButtons.Left) // Left
            {
                int index = int.Parse(((Button)sender).Name.Split('_')[1]); // Get control number from name.
                if ((isEncoderAdjActive == true) && ((index == 0) || (index >= 6)))
                {
                    return;
                }
                setSwitchPush(index);
                System.Threading.Thread.Sleep(50);
                getLEDandUpdate();
                if ((index >= 1) && (index <= 5))
                {
                    isEncoderAdjActive = true;
                }
                label_LCD.Text = getLcdString();
            }
        }

        private void button_MouseUp(object sender, MouseEventArgs e)
        {
            if (serialPort_aFrame.IsOpen == false)
            {
                MessageBox.Show("Check connection !!", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (e.Button == MouseButtons.Left) // Left
            {
                int index = int.Parse(((Button)sender).Name.Split('_')[1]); // Get control number from name.
                if ((isEncoderAdjActive == true) && ((index == 0) || (index >= 6)))
                {
                    return;
                }
                setSwitchRelease(index);
                System.Threading.Thread.Sleep(50);
                getLEDandUpdate();
                if ((index >= 1) && (index <= 5))
                {
                    isEncoderAdjActive = false;
                }
                label_LCD.Text = getLcdString();
            }
        }

        private void button_Encoder_down_MouseDown(object sender, MouseEventArgs e)
        {
            if (serialPort_aFrame.IsOpen == false)
            {
                MessageBox.Show("Check connection !!", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if ((isEncoderAdjActive == true) && (e.Button == MouseButtons.Left))
            {
                setEncoderValue(0); // down
                label_LCD.Text = getLcdString();
            }
        }

        private void button_Encoder_up_MouseDown(object sender, MouseEventArgs e)
        {
            if (serialPort_aFrame.IsOpen == false)
            {
                MessageBox.Show("Check connection !!", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if ((isEncoderAdjActive == true) && (e.Button == MouseButtons.Left))
            {
                setEncoderValue(1); // up
                label_LCD.Text = getLcdString();
            }
        }

        private void button_12_MouseDown(object sender, MouseEventArgs e)
        {
            if (serialPort_aFrame.IsOpen == false)
            {
                MessageBox.Show("Check connection !!", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if ((isEncoderAdjActive == false) && (e.Button == MouseButtons.Left))
            {
                int index = int.Parse(((Button)sender).Name.Split('_')[1]); // Get control number from name.
                setSwitchPush(index);
                label_LCD.Text = getLcdString();
            }
        }

        private void button_12_MouseUp(object sender, MouseEventArgs e)
        {
            if (serialPort_aFrame.IsOpen == false)
            {
                MessageBox.Show("Check connection !!", "ERROR", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if ((isEncoderAdjActive == false) && (e.Button == MouseButtons.Left))
            {
                int index = int.Parse(((Button)sender).Name.Split('_')[1]); // Get control number from name.
                setSwitchRelease(index);
                label_LCD.Text = getLcdString();
            }
        }

        /***** Timer event *****/
        private void timer_sync_Tick(object sender, EventArgs e)
        {
            if (isDevFound == false)
            {
                label_LCD.Text = "aFrame is not connected.";
                InitComPort();
                return;
            }

            if (serialPort_aFrame.IsOpen == false)
                isSerialConnected = false;

            if (isSerialConnected == true)
            {
                getLEDandUpdate();
                int swPat = getSwitch() & 0xF;
                if ((swPatternPre == 0x0) && (swPat != 0x0)) // Off => On
                {
                    isEncoderAdjActive = true;
                }
                if ((swPatternPre != 0x0) && (swPat == 0x0)) // On => Off
                {
                    isEncoderAdjActive = false;
                }
                label_LCD.Text = getLcdString();
                swPatternPre = swPat;
            }
            else
            {
                label_Version.Text = "VER.X.XX";
                for (int i = 1; i < swCollection.Length; i++)
                {
                    if (i == 11)
                        continue;
                    swCollection[i].BackColor = Color.Transparent;
                }
                label_peak_center.BackColor = Color.Transparent;
                label_peak_edge.BackColor = Color.Transparent;
                prjGrpList.Clear();
                prjToneDataList.instToneData.Clear();
                prjToneDataList.effectToneData.Clear();
                prjToneNameList.instName.Clear();
                prjToneNameList.effectName.Clear();
                toolStripButton_Discon.Enabled = false;
                toolStripButton_Con.Enabled = false;
                toolStripButton_Print.Enabled = false;
                isDevFound = false;
            }
        }

        private void OnElapsedTimer(Object sender, ElapsedEventArgs e) // Timer Event Hnadler
        {
#if DEBUG
            MessageBox.Show("[ERR] Sorry, Timeout Occured!! ");
#endif
            isSerialTimerout = true;
        }

        /***** FormClosed *****/
        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            disconnectComPort();
        }
    }

    /******* Other class define ***************/
    class ProjectGroupList
    {
        public int gMax;
        public List<int> instNum;
        public List<int> effectNum;

        public ProjectGroupList()
        {
            instNum = new List<int>();
            effectNum = new List<int>();
        }

    }

    class ProjectToneNameList
    {
        public List<string> instName;
        public List<string> effectName;

        public ProjectToneNameList()
        {
            instName = new List<string>();
            effectName = new List<string>();
        }
    }

    class ProjectToneDataList
    {
        public List<ProjectToneData> instToneData;
        public List<ProjectToneData> effectToneData;

        public ProjectToneDataList()
        {
            instToneData = new List<ProjectToneData>();
            effectToneData = new List<ProjectToneData>();
        }
    }

    class ProjectToneData
    {
        public int algo = 0;
        public string name = "";
        public int pNum = 0; // Number of Parameter.
        public List<int> pVal;

        public ProjectToneData()
        {
            pVal = new List<int>();
        }
    }
}
